package cn.com.poc.expose.aggregate.impl;

import cn.com.poc.agent_application.aggregate.AgentApplicationInfoService;
import cn.com.poc.agent_application.constant.AgentApplicationConstants;
import cn.com.poc.agent_application.constant.AgentApplicationDialoguesRecordConstants;
import cn.com.poc.agent_application.entity.BizAgentApplicationDialoguesRecordEntity;
import cn.com.poc.agent_application.entity.BizAgentApplicationPublishEntity;
import cn.com.poc.agent_application.service.BizAgentApplicationDialoguesRecordService;
import cn.com.poc.agent_application.service.BizAgentApplicationPublishService;
import cn.com.poc.common.utils.BlContext;
import cn.com.poc.common.utils.JsonUtils;
import cn.com.poc.expose.aggregate.AgentApplicationService;
import cn.com.poc.expose.dto.AgentApplicationDto;
import cn.com.poc.expose.rest.AgentApplicationRest;
import cn.com.poc.support.security.oauth.entity.UserBaseEntity;
import cn.com.poc.thirdparty.resource.demand.ai.common.domain.Message;
import cn.com.poc.thirdparty.resource.demand.ai.common.domain.MultiContent;
import cn.com.yict.framemax.core.exception.BusinessException;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

@Service
public class AgentApplicationServiceImpl implements AgentApplicationService {

    final private Logger logger = LoggerFactory.getLogger(AgentApplicationService.class);
    @Resource
    private BizAgentApplicationPublishService bizAgentApplicationPublishService;

    @Resource
    private BizAgentApplicationDialoguesRecordService bizAgentApplicationDialoguesRecordService;

    @Resource
    private AgentApplicationInfoService agentApplicationInfoService;


    @Override
    public void callAgentApplication(String agentId, String dialogsId, String input, HttpServletResponse httpServletResponse) throws Exception {

        UserBaseEntity userBaseEntity = BlContext.getCurrentUserNotException();
        if (userBaseEntity == null) {
            throw new BusinessException("用户未登录");
        }

        // 记录输入时间戳
        Long inputTimestamp = System.currentTimeMillis();

        BizAgentApplicationPublishEntity infoEntity = bizAgentApplicationPublishService.getByAgentId(agentId);
        if (infoEntity == null) {
            logger.warn("can not find agent application , agent_id:{}", agentId);
            throw new BusinessException("未找到应用");
        }

        // 构造对话参数
        List<Message> messages = new ArrayList<>();
        buildMessages(dialogsId, agentId, userBaseEntity.getUserId(), messages, input);


        String output = agentApplicationInfoService.callAgentApplication(infoEntity.getLargeModel(), infoEntity.getUnitIds()
                , infoEntity.getAgentSystem(), infoEntity.getKnowledgeIds(), infoEntity.getCommunicationTurn(), infoEntity.getTopP()
                , messages, httpServletResponse);


        //保存对话记录
        // 例句回答时间戳
        Long outputTimestamp = System.currentTimeMillis();

        BizAgentApplicationDialoguesRecordEntity inputRecord = new BizAgentApplicationDialoguesRecordEntity();
        inputRecord.setAgentId(infoEntity.getAgentId());
        inputRecord.setMemberId(userBaseEntity.getUserId());
        inputRecord.setContent(input);
        inputRecord.setDialogsId(dialogsId);
        inputRecord.setRole(AgentApplicationDialoguesRecordConstants.ROLE.USER);
        inputRecord.setTimestamp(inputTimestamp);

        BizAgentApplicationDialoguesRecordEntity outputRecord = new BizAgentApplicationDialoguesRecordEntity();
        outputRecord.setRole(AgentApplicationDialoguesRecordConstants.ROLE.ASSISTANT);
        outputRecord.setAgentId(infoEntity.getAgentId());
        outputRecord.setDialogsId(dialogsId);
        outputRecord.setMemberId(userBaseEntity.getUserId());
        outputRecord.setContent(output);
        outputRecord.setTimestamp(outputTimestamp);

        bizAgentApplicationDialoguesRecordService.save(inputRecord);
        bizAgentApplicationDialoguesRecordService.save(outputRecord);
    }

    private void buildMessages(String dialogsId, String agentId, Long userId, List<Message> messages, String input) throws Exception {
        BizAgentApplicationDialoguesRecordEntity recordEntity = new BizAgentApplicationDialoguesRecordEntity();
        recordEntity.setDialogsId(dialogsId);
        recordEntity.setMemberId(userId);
        recordEntity.setAgentId(agentId);
        List<BizAgentApplicationDialoguesRecordEntity> recordEntities = bizAgentApplicationDialoguesRecordService.findByExample(recordEntity, null);
        if (CollectionUtils.isNotEmpty(recordEntities)) {
            for (BizAgentApplicationDialoguesRecordEntity entity : recordEntities) {
                Message message = new Message();
                MultiContent multiContent = new MultiContent();
                multiContent.setType("text");
                multiContent.setText(entity.getContent());
                message.setContent(new ArrayList<MultiContent>() {{
                    add(multiContent);
                }});
                message.setRole(entity.getRole());
                messages.add(message);
            }
        }
        Message message = new Message();
        MultiContent multiContent = new MultiContent();
        multiContent.setType("text");
        multiContent.setText(input);
        message.setContent(new ArrayList<MultiContent>() {{
            add(multiContent);
        }});
        message.setRole(AgentApplicationDialoguesRecordConstants.ROLE.USER);
        messages.add(message);
        logger.info("--------- Build Messages dialogsId:{},agentId:{},messages:{}--------------", dialogsId, agentId, messages);
    }

}
